@@ -25,8 +25,9 @@ isc_error_unexpected(const char *file, int line, const char *format, ...) {
         char errbuf[256];
         static int unexpected_error_cnt = 0;
 
-        if (unexpected_error_cnt >= MAX_UNEXPECTED_ERRORS)
+        if (unexpected_error_cnt >= MAX_UNEXPECTED_ERRORS) {
                 return; /* avoid clutter in log */
+}
 
         va_start(args, format);
 
