@@ -8,7 +8,7 @@
  * Some parts borrowed from the older ntp_config.c
  * Copyright (c) 2006
  * Copyright 2015 by the NTPsec project contributors
- * SPDX-License-Identifier: BSD-2-clause
+ * SPDX-License-Identifier: BSD-2-Clause
 */
 
 #include "config.h"
@@ -209,6 +209,7 @@ static void free_config_rlimit(config_tree *);
 static void free_config_setvar(config_tree *);
 static void free_config_system_opts(config_tree *);
 static void free_config_tinker(config_tree *);
+static void free_config_nts(config_tree *);
 static void free_config_tos(config_tree *);
 static void free_config_unpeers(config_tree *);
 static void free_config_vars(config_tree *);
@@ -266,6 +267,7 @@ static void config_monitor(config_tree *);
 static void config_rlimit(config_tree *);
 static void config_system_opts(config_tree *);
 static void config_tinker(config_tree *);
+static void config_nts(config_tree *);
 static void config_tos(config_tree *);
 static void config_logfile(config_tree *);
 static void config_vars(config_tree *);
@@ -355,6 +357,7 @@ free_config_tree(
 	free_config_monitor(ptree);
 	free_config_access(ptree);
 	free_config_tinker(ptree);
+	free_config_nts(ptree);
 	free_config_rlimit(ptree);
 	free_config_system_opts(ptree);
 	free_config_logconfig(ptree);
@@ -502,8 +505,9 @@ create_attr_sval(
 
 	my_val = emalloc_zero(sizeof(*my_val));
 	my_val->attr = attr;
-	if (NULL == s)			/* free() hates NULL */
+	if (NULL == s) {			/* free() hates NULL */
 		s = estrdup("");
+	}
 	my_val->value.s = _UC(s);
 	my_val->type = T_String;
 
@@ -628,6 +632,14 @@ create_peer_node(
 				my_node->ctl.flags |= FLAG_NOSELECT;
 				break;
 
+			case T_Noval:
+				my_node->ctl.flags |= FLAG_NTS_NOVAL;
+				break;
+
+			case T_Nts:
+				my_node->ctl.flags |= (FLAG_NTS | FLAG_LOOKUP);
+				break;
+
 			case T_Prefer:
 				my_node->ctl.flags |= FLAG_PREFER;
 				break;
@@ -638,6 +650,22 @@ create_peer_node(
 			}
 			break;
 
+		case T_Expire:
+		    my_node->ctl.nts_cfg.expire = option->value.u;
+		    break;
+
+		case T_Aead:
+		    my_node->ctl.nts_cfg.aead = option->value.s;
+		    break;
+
+		case T_Ca:
+		    my_node->ctl.nts_cfg.ca = option->value.s;
+		    break;
+
+		case T_Cert:
+		    my_node->ctl.nts_cfg.cert = option->value.s;
+		    break;
+
 		case T_Minpoll:
 			if (option->value.i < NTP_MINPOLL ) {
 				msyslog(LOG_INFO,
@@ -705,6 +733,16 @@ create_peer_node(
 			my_node->ctl.bias = option->value.d;
 			break;
 
+		case T_Ask:
+			my_node->ctl.flags |= FLAG_NTS_ASK;
+			my_node->ctl.nts_cfg.server = estrdup(option->value.s);
+			break;
+
+		case T_Require:
+			my_node->ctl.flags |= FLAG_NTS_REQ;
+			my_node->ctl.nts_cfg.server = estrdup(option->value.s);
+			break;
+
 #ifdef REFCLOCK
 		case T_Path:
 			my_node->ctl.path = estrdup(option->value.s);
@@ -820,8 +858,9 @@ create_unpeer_node(
 	 * We treat all valid 16-bit numbers as association IDs.
 	 */
 	pch = addr->address;
-	while (*pch && isdigit((unsigned char)*pch))
+	while (*pch && isdigit((unsigned char)*pch)) {
 		pch++;
+	}
 
 	if (!*pch
 	    && 1 == sscanf(addr->address, "%u", &u)
@@ -1043,8 +1082,9 @@ create_setvar_node(
 
 	/* do not allow = in the variable name */
 	pch = strchr(var, '=');
-	if (NULL != pch)
+	if (NULL != pch) {
 		*pch = '\0';
+}
 
 	/* Now store the string into a setvar_node */
 	my_node = emalloc_zero(sizeof(*my_node));
@@ -1108,8 +1148,9 @@ config_auth(
 
 	/* ntp_signd_socket Command */
 	if (ptree->auth.ntp_signd_socket) {
-		if (ntp_signd_socket != default_ntp_signd_socket)
+		if (ntp_signd_socket != default_ntp_signd_socket) {
 			free(ntp_signd_socket);
+		}
 		ntp_signd_socket = estrdup(ptree->auth.ntp_signd_socket);
 	}
 
@@ -1286,7 +1327,7 @@ config_monitor(
 
 	/* NOTE:
 	 * Calling filegen_get is brain dead. Doing a string
-	 * comparison to find the relavant filegen structure is
+	 * comparison to find the relevant filegen structure is
 	 * expensive.
 	 *
 	 * Through the parser, we already know which filegen is
@@ -1552,7 +1593,7 @@ config_access(
 		case T_Average:
 			if (0 <= my_opt->value.i &&
 			    my_opt->value.i <= UCHAR_MAX)
-				ntp_minpoll = (uint8_t)my_opt->value.u;
+				rstrct.ntp_minpoll = (uint8_t)my_opt->value.u;
 			else
 				msyslog(LOG_ERR,
 					"CONFIG: discard average %d out of range, ignored.",
@@ -1560,7 +1601,7 @@ config_access(
 			break;
 
 		case T_Minimum:
-			ntp_minpkt = my_opt->value.i;
+			rstrct.ntp_minpkt = my_opt->value.i;
 			break;
 
 		case T_Monitor:
@@ -1945,6 +1986,68 @@ config_tinker(
 	}
 }
 
+static void
+config_nts(
+	config_tree *ptree
+	)
+{
+	attr_val *	nts;
+
+	nts = HEAD_PFIFO(ptree->nts);
+	for (; nts != NULL; nts = nts->link) {
+		switch (nts->attr) {
+
+		default:
+			INSIST(0);
+			break;
+
+		case T_Aead:
+			ntsconfig.aead = estrdup(nts->value.s);
+			break;
+
+		case T_Ca:
+			ntsconfig.ca = estrdup(nts->value.s);
+			break;
+
+		case T_Cert:
+			ntsconfig.cert = estrdup(nts->value.s);
+			break;
+
+		case T_Cookie:
+			ntsconfig.KI = estrdup(nts->value.s);
+			break;
+
+		case T_Disable:
+			ntsconfig.ntsenable = false;
+			break;
+
+		case T_Enable:
+			ntsconfig.ntsenable = true;
+			break;
+
+		case T_Key:
+			ntsconfig.key = estrdup(nts->value.s);
+			break;
+
+		case T_Maxtls:
+			ntsconfig.maxtls = estrdup(nts->value.s);
+			break;
+
+		case T_Mintls:
+			ntsconfig.mintls = estrdup(nts->value.s);
+			break;
+
+		case T_Tlsciphers:
+			ntsconfig.tlsciphers = estrdup(nts->value.s);
+			break;
+
+		case T_Tlsciphersuites:
+			ntsconfig.tlsciphersuites = estrdup(nts->value.s);
+			break;
+		}
+	}
+}
+
 
 static void
 free_config_rlimit(
@@ -1963,6 +2066,15 @@ free_config_tinker(
 }
 
 
+static void
+free_config_nts(
+	config_tree *ptree
+	)
+{
+	FREE_ATTR_VAL_FIFO(ptree->nts);
+}
+
+
 /*
  * config_nic_rules - apply interface listen/ignore/drop items
  */
@@ -2278,8 +2390,9 @@ config_setvar(
 						? DEF
 						: 0);
 	}
-	if (str != NULL)
+	if (str != NULL) {
 		free(str);
+	}
 }
 
 
@@ -2301,18 +2414,16 @@ config_fudge(
 # ifdef REFCLOCK
 	attr_val *curr_opt;
 	sockaddr_u addr_sock;
-	address_node *addr_node;
 	struct refclockstat clock_stat;
-	bool err_flag;
 
 	curr_fudge = HEAD_PFIFO(ptree->fudge);
 	for (; curr_fudge != NULL; curr_fudge = curr_fudge->link) {
-		err_flag = false;
+		bool err_flag = false;
 
 		/* Get the reference clock address and
 		 * ensure that it is sane
 		 */
-		addr_node = curr_fudge->addr;
+		address_node *addr_node = curr_fudge->addr;
 		ZERO_SOCK(&addr_sock);
 		if (getnetnum(addr_node->address, &addr_sock)
 		    != 1) {
@@ -2434,8 +2545,8 @@ config_logfile(
 		case T_Logfile:
 			if (-1 == change_logfile(curr_var->value.s, true))
 				msyslog(LOG_ERR,
-					"CONFIG: Cannot open logfile %s: %m",
-					curr_var->value.s);
+					"CONFIG: Cannot open logfile %s: %s",
+					curr_var->value.s, strerror(errno));
 			break;
 
 		default:
@@ -2572,13 +2683,6 @@ peer_config(
 	uint8_t cast_flags;
 	uint8_t hmode;
 
-#ifndef ENABLE_DNS_LOOKUP
-	if (NULL != hostname) {
-		msyslog(LOG_ERR, "CONFIG: hostnames need DNS lookup: %s", hostname);
-		return NULL;
-	}
-#endif
-
 	/*
 	 * We do a dirty little jig to figure the cast flags. This is
 	 * probably not the best place to do this, at least until the
@@ -2600,7 +2704,7 @@ peer_config(
 		cast_flags = MDF_UCAST;
 		hmode = MODE_CLIENT;
 		if (NULL != hostname)
-			ctl->flags |= FLAG_DNS;
+			ctl->flags |= (FLAG_DNS | FLAG_LOOKUP);
 		break;
 
 	default:
@@ -2629,145 +2733,144 @@ config_peers(
 	config_tree *ptree
 	)
 {
-    sockaddr_u		peeraddr;
-    peer_node *		curr_peer;
+	sockaddr_u		peeraddr;
 
-    /* add servers named on the command line with iburst implied */
-    for ( ; cmdline_server_count > 0;
-	cmdline_server_count--, cmdline_servers++) {
-	     struct peer_ctl client_ctl;
-	     /* so any new members we introduce will be zeroed */
-	     memset(&client_ctl, '\0', sizeof(struct peer_ctl));
-	     client_ctl.version = NTP_VERSION;
-	     client_ctl.minpoll = NTP_MINDPOLL;
-	     client_ctl.maxpoll = NTP_MAXPOLL_UNK;
-	     client_ctl.flags = FLAG_IBURST;
-	     client_ctl.mode = 0;
-	     client_ctl.peerkey = 0;
-	     client_ctl.bias = 0;
+	/* add servers named on the command line with iburst implied */
+	for ( ; cmdline_server_count > 0;
+	      cmdline_server_count--, cmdline_servers++) {
+		struct peer_ctl client_ctl;
+		/* so any new members we introduce will be zeroed */
+		memset(&client_ctl, '\0', sizeof(struct peer_ctl));
+		client_ctl.version = NTP_VERSION;
+		client_ctl.minpoll = NTP_MINDPOLL;
+		client_ctl.maxpoll = NTP_MAXPOLL_UNK;
+		client_ctl.flags = FLAG_IBURST;
+		client_ctl.mode = 0;
+		client_ctl.peerkey = 0;
+		client_ctl.bias = 0;
 
-	    ZERO_SOCK(&peeraddr);
-	    /*
-	     * If we have a numeric address, we can safely
-	     * proceed in the mainline with it.  Otherwise, hand
-	     * the hostname off to the blocking child.
-	     */
-	    if (is_ip_address(*cmdline_servers, AF_UNSPEC, &peeraddr)) {
-		SET_PORT(&peeraddr, NTP_PORT);
-		if (is_sane_resolved_address(&peeraddr,
-					     T_Server))
+		ZERO_SOCK(&peeraddr);
+		/*
+		 * If we have a numeric address, we can safely
+		 * proceed in the mainline with it.  Otherwise, hand
+		 * the hostname off to the blocking child.
+		 */
+		if (is_ip_address(*cmdline_servers, AF_UNSPEC, &peeraddr)) {
+			SET_PORT(&peeraddr, NTP_PORT);
+			if (is_sane_resolved_address(&peeraddr,
+						     T_Server))
+				peer_config(
+					&peeraddr,
+					NULL,
+					NULL,
+					T_Server,
+					&client_ctl);
+		} else {
+			/* we have a hostname to resolve */
 			peer_config(
 				&peeraddr,
-				NULL,
+				*cmdline_servers,
 				NULL,
 				T_Server,
 				&client_ctl);
-	    } else {
-		/* we have a hostname to resolve */
-		peer_config(
-			&peeraddr,
-			*cmdline_servers,
-			NULL,
-			T_Server,
-			&client_ctl);
-	    }
-    }
-
-    /* add associations from the configuration file */
-    curr_peer = HEAD_PFIFO(ptree->peers);
-    for (; curr_peer != NULL; curr_peer = curr_peer->link) {
-	ZERO_SOCK(&peeraddr);
-
-	if (T_Pool == curr_peer->host_mode) {
-	    AF(&peeraddr) = curr_peer->addr->type;
-	    peer_config(
-		    &peeraddr,
-		    curr_peer->addr->address,
-		    NULL,
-		    curr_peer->host_mode,
-		    &curr_peer->ctl);
-	/*
-	 * If we have a numeric address, we can safely
-	 * proceed in the mainline with it.
-	 */
-	} else if (is_ip_address(curr_peer->addr->address,
-			  curr_peer->addr->type, &peeraddr)) {
-
-	    SET_PORT(&peeraddr, NTP_PORT);
-	    if (is_sane_resolved_address(&peeraddr, curr_peer->host_mode)) {
-#ifdef REFCLOCK
-		/* save maxpoll from config line
-		 * newpeer smashes it
-		 */
-		uint8_t maxpoll = curr_peer->ctl.maxpoll;
-#endif
-		struct peer *peer = peer_config(
-			&peeraddr,
-			NULL,
-			NULL,
-			curr_peer->host_mode,
-			&curr_peer->ctl);
-		if ( NULL == peer )
-		{
-		    /* duplicate peer !?, ignore */
-		    msyslog(LOG_INFO, "CONFIG: configpeers: Ignoring duplicate '%s'",
-			socktoa(&peeraddr));
-		    continue;
 		}
-		if (ISREFCLOCKADR(&peeraddr))
-		{
-#ifdef REFCLOCK
-		    uint8_t clktype;
-		    int unit;
-		    /*
-		     * We let the reference clock
-		     * support do clock dependent
-		     * initialization.  This
-		     * includes setting the peer
-		     * timer, since the clock may
-		     * have requirements for this.
-		     */
-		    if (NTP_MAXPOLL_UNK == maxpoll)
-			    /* default maxpoll for
-			     * refclocks is minpoll
-			     */
-			    peer->cfg.maxpoll = peer->cfg.minpoll;
-		    clktype = (uint8_t)REFCLOCKTYPE(&peer->srcadr);
-		    unit = REFCLOCKUNIT(&peer->srcadr);
-
-		    peer->cfg.path = curr_peer->ctl.path;
-		    peer->cfg.ppspath = curr_peer->ctl.ppspath;
-		    peer->cfg.baud = curr_peer->ctl.baud;
-		    if (refclock_newpeer(clktype,
-					  unit,
-					  peer))
-			    refclock_control(&peeraddr,
-					     &curr_peer->clock_stat,
-					     NULL);
-		    else
-			    /*
-			     * Dump it, something screwed up
-			     */
-			    unpeer(peer);
-#else /* REFCLOCK */
-		    msyslog(LOG_ERR,
-			 "INIT: ntpd was compiled without refclock support.");
-		    unpeer(peer);
-#endif /* REFCLOCK */
-		}
-
-	    }
-	/* DNS lookup */
-	} else {
-	    AF(&peeraddr) = curr_peer->addr->type;
-	    peer_config(
-		&peeraddr,
-		curr_peer->addr->address,
-		NULL,
-		curr_peer->host_mode,
-		&curr_peer->ctl);
 	}
-    }
+
+	/* add associations from the configuration file */
+	peer_node * curr_peer = HEAD_PFIFO(ptree->peers);
+	for (; curr_peer != NULL; curr_peer = curr_peer->link) {
+		ZERO_SOCK(&peeraddr);
+
+		if (T_Pool == curr_peer->host_mode) {
+			AF(&peeraddr) = curr_peer->addr->type;
+			peer_config(
+				&peeraddr,
+				curr_peer->addr->address,
+				NULL,
+				curr_peer->host_mode,
+				&curr_peer->ctl);
+			/*
+			 * If we have a numeric address, we can safely
+			 * proceed in the mainline with it.
+			 */
+		} else if (is_ip_address(curr_peer->addr->address,
+					 curr_peer->addr->type, &peeraddr)) {
+
+			SET_PORT(&peeraddr, NTP_PORT);
+			if (is_sane_resolved_address(&peeraddr, curr_peer->host_mode)) {
+#ifdef REFCLOCK
+				/* save maxpoll from config line
+				 * newpeer smashes it
+				 */
+				uint8_t maxpoll = curr_peer->ctl.maxpoll;
+#endif
+				struct peer *peer = peer_config(
+					&peeraddr,
+					NULL,
+					NULL,
+					curr_peer->host_mode,
+					&curr_peer->ctl);
+				if ( NULL == peer )
+				{
+					/* duplicate peer !?, ignore */
+					msyslog(LOG_INFO, "CONFIG: configpeers: Ignoring duplicate '%s'",
+						socktoa(&peeraddr));
+					continue;
+				}
+				if (ISREFCLOCKADR(&peeraddr))
+				{
+#ifdef REFCLOCK
+					uint8_t clktype;
+					int unit;
+					/*
+					 * We let the reference clock
+					 * support do clock dependent
+					 * initialization.  This
+					 * includes setting the peer
+					 * timer, since the clock may
+					 * have requirements for this.
+					 */
+					if (NTP_MAXPOLL_UNK == maxpoll)
+						/* default maxpoll for
+						 * refclocks is minpoll
+						 */
+						peer->cfg.maxpoll = peer->cfg.minpoll;
+					clktype = (uint8_t)REFCLOCKTYPE(&peer->srcadr);
+					unit = REFCLOCKUNIT(&peer->srcadr);
+
+					peer->cfg.path = curr_peer->ctl.path;
+					peer->cfg.ppspath = curr_peer->ctl.ppspath;
+					peer->cfg.baud = curr_peer->ctl.baud;
+					if (refclock_newpeer(clktype,
+							     unit,
+							     peer))
+						refclock_control(&peeraddr,
+								 &curr_peer->clock_stat,
+								 NULL);
+					else
+						/*
+						 * Dump it, something screwed up
+						 */
+						unpeer(peer);
+#else /* REFCLOCK */
+					msyslog(LOG_ERR,
+						"INIT: ntpd was compiled without refclock support.");
+					unpeer(peer);
+#endif /* REFCLOCK */
+				}
+
+			}
+			/* DNS lookup */
+		} else {
+			AF(&peeraddr) = curr_peer->addr->type;
+			peer_config(
+				&peeraddr,
+				curr_peer->addr->address,
+				NULL,
+				curr_peer->host_mode,
+				&curr_peer->ctl);
+		}
+	}
 }
 
 static void
@@ -2797,12 +2900,11 @@ config_unpeers(
 	)
 {
 	sockaddr_u		peeraddr;
-	unpeer_node *		curr_unpeer;
 	struct peer *		p;
 	const char *		name;
 	int			rc;
 
-	curr_unpeer = HEAD_PFIFO(ptree->unpeers);
+	unpeer_node * curr_unpeer = HEAD_PFIFO(ptree->unpeers);
 	for (; curr_unpeer != NULL; curr_unpeer = curr_unpeer->link) {
 		/*
 		 * Either AssocID will be zero, and we unpeer by name/
@@ -2951,6 +3053,7 @@ config_ntpd(
 	config_tos(ptree);
 	config_access(ptree);
 	config_tinker(ptree);
+	config_nts(ptree);
 	config_rlimit(ptree);
 	config_system_opts(ptree);
 	config_logconfig(ptree);
@@ -3098,7 +3201,7 @@ void readconfig(const char *config_file)
 void set_keys_file(char* keys)
 {
 	cfgt.auth.keys = estrdup(keys);
-};
+}
 
 void set_trustedkey(keyid_t tkey)
 {
@@ -3106,7 +3209,7 @@ void set_trustedkey(keyid_t tkey)
 	attr_val *val2 = NULL;
 	APPEND_G_FIFO(val2, val);
 	CONCAT_G_FIFOS(cfgt.auth.trusted_key_list, val2);
-};
+}
 
 
 
@@ -3177,10 +3280,11 @@ get_match(
 	)
 {
 	while (m->name != NULL) {
-		if (strcmp(str, m->name) == 0)
+		if (strcmp(str, m->name) == 0) {
 			return m->mask;
-		else
+		} else {
 			m++;
+		}
 	}
 	return 0;
 }
@@ -3399,84 +3503,84 @@ static void
 fix_node_cidr(
     restrict_node *my_node)
 {
-    address_node *addr;
-    char mask_s[40], *mask_p;
-    char *cidr_p;
-    char *colon_p;
-    char *endptr;
-    long cidr_len;
-    int i;
-    unsigned a[8];
+	address_node *addr;
+	char mask_s[40], *mask_p;
+	char *cidr_p;
+	char *colon_p;
+	char *endptr;
+	long cidr_len;
+	int i;
+	unsigned a[8];
 
-    REQUIRE(my_node);
-    addr = my_node->addr;
-    REQUIRE(addr);
+	REQUIRE(my_node);
+	addr = my_node->addr;
+	REQUIRE(addr);
 
-    cidr_p = strrchr(addr->address, '/');
-    if (!cidr_p) {
-        /* not CIDR, leave silently */
-        return;
-    }
-    *cidr_p++ = '\0'; /* remove the '/' and beyond from address */
-    /* get CIDR as int */
-    errno = 0;
-    cidr_len = strtol(cidr_p, &endptr, 10);
-    if ( errno || (endptr == cidr_p) ) {
-        /* conversion fail, leave silently */
-        return;
-    }
-    if ( 0 >= cidr_len ) {
-        /* negative or zero?  leave silently */
-	/* exiting on 0 avoids a bad shift warning from Coverity */
-        return;
-    }
-    /* sadly, addr->type not previously set, look for colon */
-    colon_p = strrchr(addr->address, ':');
-    if (colon_p) {
-        /* IPv6 */
-        uint64_t mask_top = 0xFFFFFFFFFFFFFFFFU;
-        uint64_t mask_bot = 0xFFFFFFFFFFFFFFFFU;
-
-	if ( 128 < cidr_len ) {
-	    /* out of range, leave silently */
-	    return;
+	cidr_p = strrchr(addr->address, '/');
+	if (!cidr_p) {
+		/* not CIDR, leave silently */
+		return;
 	}
-        if ( 64 >= cidr_len ) {
-	    mask_bot = 0;
-	    mask_top <<= 64 - cidr_len ;
-        } else {
-	    mask_bot <<= 128 - cidr_len ;
-        }
-        for (i = 0; i < 4; i++)
-                a[i] = mask_top >> (16 * (3 - i)) & 0xffffU;
-        for (i = 0; i < 4; i++)
-                a[i + 4] = mask_bot >> (16 * (3 - i)) & 0xffffU;
+	*cidr_p++ = '\0'; /* remove the '/' and beyond from address */
+	/* get CIDR as int */
+	errno = 0;
+	cidr_len = strtol(cidr_p, &endptr, 10);
+	if ( errno || (endptr == cidr_p) ) {
+		/* conversion fail, leave silently */
+		return;
+	}
+	if ( 0 >= cidr_len ) {
+		/* negative or zero?  leave silently */
+		/* exiting on 0 avoids a bad shift warning from Coverity */
+		return;
+	}
+	/* sadly, addr->type not previously set, look for colon */
+	colon_p = strrchr(addr->address, ':');
+	if (colon_p) {
+		/* IPv6 */
+		uint64_t mask_top = 0xFFFFFFFFFFFFFFFFU;
+		uint64_t mask_bot = 0xFFFFFFFFFFFFFFFFU;
 
-        snprintf(mask_s, sizeof(mask_s), "%x:%x:%x:%x:%x:%x:%x:%x",
-                 a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7]);
-    } else {
-        /* must be IPv4 */
-        uint32_t mask_n = 0xFFFFFFFFU;
+		if ( 128 < cidr_len ) {
+			/* out of range, leave silently */
+			return;
+		}
+		if ( 64 >= cidr_len ) {
+			mask_bot = 0;
+			mask_top <<= 64 - cidr_len ;
+		} else {
+			mask_bot <<= 128 - cidr_len ;
+		}
+		for (i = 0; i < 4; i++)
+			a[i] = mask_top >> (16 * (3 - i)) & 0xffffU;
+		for (i = 0; i < 4; i++)
+			a[i + 4] = mask_bot >> (16 * (3 - i)) & 0xffffU;
 
-	if ( 32 < cidr_len ) {
-	    /* out of range, leave silently */
-	    return;
+		snprintf(mask_s, sizeof(mask_s), "%x:%x:%x:%x:%x:%x:%x:%x",
+			 a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7]);
+	} else {
+		/* must be IPv4 */
+		uint32_t mask_n = 0xFFFFFFFFU;
+
+		if ( 32 < cidr_len ) {
+			/* out of range, leave silently */
+			return;
+		}
+
+		mask_n <<= 32 - cidr_len ;
+		for (i = 0; i < 4; i++)
+			a[i] = mask_n >> (8 * (3 - i)) & 0xff;
+
+		snprintf(mask_s, sizeof(mask_s), "%u.%u.%u.%u",
+			 a[0], a[1], a[2], a[3]);
 	}
 
-        mask_n <<= 32 - cidr_len ;
-        for (i = 0; i < 4; i++)
-                a[i] = mask_n >> (8 * (3 - i)) & 0xff;
+	/* lose old mask */
+	destroy_address_node(my_node->mask);
 
-        snprintf(mask_s, sizeof(mask_s), "%u.%u.%u.%u",
-                 a[0], a[1], a[2], a[3]);
-    }
-
-    /* lose old mask */
-    destroy_address_node(my_node->mask);
-
-    /* create mask node, yes AF_UNSPEC is weird... */
-    mask_p = estrdup(mask_s);
-    my_node->mask = create_address_node(mask_p, AF_UNSPEC);
+	/* create mask node, yes AF_UNSPEC is weird... */
+	mask_p = estrdup(mask_s);
+	my_node->mask = create_address_node(mask_p, AF_UNSPEC);
 }
 
 /*
@@ -3544,7 +3648,7 @@ ntp_rlimit(
 			   (int)rl_value / rl_scale, rl_sstr));
 		rl.rlim_cur = rl.rlim_max = rl_value;
 		if (setrlimit(RLIMIT_NOFILE, &rl) == -1)
-			msyslog(LOG_ERR, "CONFIG: Cannot set RLIMIT_NOFILE: %m");
+			msyslog(LOG_ERR, "CONFIG: Cannot set RLIMIT_NOFILE: %s", strerror(errno));
 		break;
 #endif /* RLIMIT_NOFILE */
 
@@ -3558,7 +3662,7 @@ ntp_rlimit(
 		DPRINT(2, ("ntp_rlimit: STACK: %d %s pages\n",
 			   (int)rl_value / rl_scale, rl_sstr));
 		if (-1 == getrlimit(RLIMIT_STACK, &rl)) {
-			msyslog(LOG_ERR, "CONFIG: getrlimit(RLIMIT_STACK) failed: %m");
+			msyslog(LOG_ERR, "CONFIG: getrlimit(RLIMIT_STACK) failed: %s", strerror(errno));
 		} else {
 			if (rl_value > rl.rlim_max) {
 				msyslog(LOG_WARNING,
@@ -3570,7 +3674,7 @@ ntp_rlimit(
 			rl.rlim_cur = rl_value;
 			if (-1 == setrlimit(RLIMIT_STACK, &rl)) {
 				msyslog(LOG_ERR,
-					"CONFIG: ntp_rlimit: Cannot set RLIMIT_STACK: %m");
+					"CONFIG: ntp_rlimit: Cannot set RLIMIT_STACK: %s", strerror(errno));
 			}
 		}
 		break;
