@@ -74,8 +74,9 @@
  * Get the little JSMN library directly into our guts. Use the 'parent
  * link' feature for maximum speed.
  */
+#define JSMN_STATIC
 #define JSMN_PARENT_LINKS
-#include "../libjsmn/jsmn.c"
+#include "jsmn.h"
 
 /* =====================================================================
  * JSON parsing stuff
@@ -166,7 +167,10 @@ typedef unsigned long int json_uint;
 #define	NAME		"GPSD"	/* shortname */
 #define	DESCRIPTION	"GPSD JSON client clock" /* who we are */
 
-#define MAX_PDU_LEN	1600
+/* MAX_PDU_LEN needs to be bigger than GPS_JSON_RESPONSE_MAX from gpsd.
+ * As of March 2019 that is 4096 */
+#define MAX_PDU_LEN	8192
+
 #define TICKOVER_LOW	10
 #define TICKOVER_HIGH	120
 #define LOGTHROTTLE	SECSPERHR
@@ -410,7 +414,7 @@ static void
 gpsd_init(void)
 {
 	addrinfoT   hints;
-	int         rc, idx;
+	int         idx;
 
 	memset(s_svcerr, 0, sizeof(s_svcerr));
 	memset(&hints, 0, sizeof(hints));
@@ -419,11 +423,12 @@ gpsd_init(void)
 	hints.ai_socktype = SOCK_STREAM;
 
 	for (idx = 0; s_svctab[idx][0] && !s_gpsd_addr; idx++) {
-		rc = getaddrinfo(s_svctab[idx][0], s_svctab[idx][1],
-				 &hints, &s_gpsd_addr);
+		int rc = getaddrinfo(s_svctab[idx][0], s_svctab[idx][1],
+				     &hints, &s_gpsd_addr);
 		s_svcerr[idx] = rc;
-		if (0 == rc)
+		if (0 == rc) {
 			break;
+		}
 		s_gpsd_addr = NULL;
 	}
 	s_svcidx = idx;
@@ -438,8 +443,9 @@ gpsd_init_check(void)
 	int idx;
 
 	/* Check if there is something to log */
-	if (s_svcidx == 0)
+	if (s_svcidx == 0) {
 		return (s_gpsd_addr != NULL);
+	}
 
 	/* spool out the resolver errors */
 	for (idx = 0; idx < s_svcidx; ++idx) {
@@ -486,8 +492,9 @@ gpsd_start(
 		return false;
 
 	/* search for matching unit */
-	while ((up = *uscan) != NULL && up->unit != (unit & 0x7F))
+	while ((up = *uscan) != NULL && up->unit != (unit & 0x7F)) {
 		uscan = &up->next_unit;
+	}
 	if (up == NULL) {
 		/* alloc unit, add to list and increment use count ASAP. */
 		up = emalloc_zero(sizeof(*up));
@@ -516,12 +523,14 @@ gpsd_start(
                 }
 		if (-1 == ret ) {
                         /* more likely out of RAM */
-			msyslog(LOG_ERR, "REFCLOCK: %s: clock device name too long",
+			msyslog(LOG_ERR,
+                                "REFCLOCK: %s: clock device name too long",
 				up->logname);
 			goto dev_fail;
 		}
 		if (-1 == stat(up->device, &sb) || !S_ISCHR(sb.st_mode)) {
-			msyslog(LOG_ERR, "REFCLOCK: %s: '%s' is not a character device",
+			msyslog(LOG_ERR,
+                                "REFCLOCK: %s: '%s' is not a character device",
 				up->logname, up->device);
 			goto dev_fail;
 		}
@@ -551,7 +560,8 @@ gpsd_start(
 
 	/* If the daemon name lookup failed, just give up now. */
 	if (NULL == up->addr) {
-		msyslog(LOG_ERR, "REFCLOCK: %s: no GPSD socket address, giving up",
+		msyslog(LOG_ERR,
+			"REFCLOCK: %s: no GPSD socket address, giving up",
 			up->logname);
 		goto dev_fail;
 	}
@@ -560,12 +570,14 @@ gpsd_start(
 	      (LOG_NOTICE, "%s: startup, device is '%s'",
 	       refclock_name(peer), up->device));
 	up->mode = MODE_OP_MODE(peer->cfg.mode);
-	if (up->mode > MODE_OP_MAXVAL)
+	if (up->mode > MODE_OP_MAXVAL) {
 		up->mode = 0;
-	if (unit >= 128)
+	}
+	if (unit >= 128) {
 		up->pps_peer = peer;
-	else
+	} else {
 		enter_opmode(peer, up->mode);
+	}
 	return true;
 
 dev_fail:
@@ -592,8 +604,9 @@ gpsd_shutdown(
 	gpsd_unitT ** uscan   = &s_clock_units;
 
 	/* The unit pointer might have been removed already. */
-	if (up == NULL)
+	if (up == NULL) {
 		return;
+	}
 
 	if (up->pps_peer == NULL) {
 		/* This is NULL if no related PPS */
@@ -606,17 +619,20 @@ gpsd_shutdown(
 			io_closeclock(&pp->io);
 			pp->io.fd = -1;
 		}
-		if (up->fdt != -1)
+		if (up->fdt != -1) {
 			close(up->fdt);
+		}
 	}
 	/* decrement use count and eventually remove this unit. */
 	if (!--up->refcount) {
 		/* unlink this unit */
-		while (*uscan != NULL)
-			if (*uscan == up)
+		while (*uscan != NULL) {
+			if (*uscan == up) {
 				*uscan = up->next_unit;
-			else
+			} else {
 				uscan = &(*uscan)->next_unit;
+			}
+		}
 		free(up->logname);
 		free(up->device);
 		free(up);
@@ -658,21 +674,23 @@ gpsd_receive(
 	pdst = up->buffer + up->buflen;
 	edst = pdst + sizeof(up->buffer) - 1; /* for trailing NUL */
 
-	while (psrc != esrc) {
+	while (psrc < esrc) {
 		ch = *psrc++;
 		if (ch == '\n') {
 			/* trim trailing whitespace & terminate buffer */
-			while (pdst != up->buffer && pdst[-1] <= ' ')
+			while (pdst != up->buffer && pdst[-1] <= ' ') {
 				--pdst;
+			}
 			*pdst = '\0';
 			/* process data and reset buffer */
 			up->buflen = (int)(pdst - up->buffer);
 			gpsd_parse(peer, &rbufp->recv_time);
 			pdst = up->buffer;
-		} else if (pdst != edst) {
+		} else if (pdst < edst) {
 			/* add next char, ignoring leading whitespace */
-			if (ch > ' ' || pdst != up->buffer)
+			if (ch > ' ' || pdst != up->buffer) {
 				*pdst++ = ch;
+			}
 		}
 	}
 	up->buflen = (int)(pdst - up->buffer);
@@ -755,10 +773,11 @@ gpsd_poll(
 	UNUSED_ARG(unit);
 
 	++pp->polls;
-	if (peer == up->pps_peer)
+	if (peer == up->pps_peer) {
 		poll_secondary(peer, pp, up);
-	else
+	} else {
 		poll_primary(peer, pp, up);
+	}
 }
 
 /* ------------------------------------------------------------------ */
@@ -812,10 +831,12 @@ timer_primary(
 	 * Note that the timer stays at zero here, unless some of the
 	 * functions set it to another value.
 	 */
-	if (up->logthrottle)
+	if (up->logthrottle) {
 		--up->logthrottle;
-	if (up->tickover)
+	}
+	if (up->tickover) {
 		--up->tickover;
+	}
 	switch (up->tickover) {
 	case 4:
 		/* If we are connected to GPSD, try to get a live signal
@@ -837,10 +858,11 @@ timer_primary(
 	case 0:
 		if (-1 != pp->io.fd)
 			gpsd_stop_socket(peer);
-		else if (-1 != up->fdt)
+		else if (-1 != up->fdt) {
 			gpsd_test_socket(peer);
-		else if (NULL != s_gpsd_addr)
+		} else if (NULL != s_gpsd_addr) {
 			gpsd_init_socket(peer);
+		}
 		break;
 
 	default:
@@ -878,10 +900,11 @@ gpsd_timer(
 
 	UNUSED_ARG(unit);
 
-	if (peer == up->pps_peer)
+	if (peer == up->pps_peer) {
 		timer_secondary(peer, pp, up);
-	else
+	} else {
 		timer_primary(peer, pp, up);
+	}
 }
 
 /* =====================================================================
@@ -1101,25 +1124,28 @@ strtojint(
 	    flags |= (accu > limit_hi);
 	}
 	/* Check for empty conversion (no digits seen). */
-	if (hold != cp)
+	if (hold != cp) {
 		vep.c = cp;
-	else
+	} else {
 		errno = EINVAL;	/* accu is still zero */
+	}
 	/* Check for range overflow */
 	if (flags & 1) {
 		errno = ERANGE;
 		accu  = limit_hi;
 	}
 	/* If possible, store back the end-of-conversion pointer */
-	if (ep)
+	if (ep) {
 		*ep = vep.v;
+	}
 	/* If negative, return the negated result if the accu is not
 	 * zero. Avoid negation overflows.
 	 */
-	if ((flags & 2) && accu)
+	if ((flags & 2) && accu) {
 		return -(json_int)(accu - 1) - 1;
-	else
+	} else {
 		return (json_int)accu;
+	}
 }
 
 /* ------------------------------------------------------------------ */
@@ -1156,8 +1182,9 @@ json_token_skip(
 			++tid;
 			break;
 		}
-		if (tid > ctx->ntok) /* Impossible? Paranoia rulez. */
+		if (tid > ctx->ntok) { /* Impossible? Paranoia rulez. */
 			tid = ctx->ntok;
+		}
 	}
 	return tid;
 }
@@ -1190,8 +1217,9 @@ json_object_lookup(
 			break;
 		}
 		/* if skipping ahead returned an error, bail out here. */
-		if (tid < 0)
+		if (tid < 0) {
 			break;
+		}
 	}
 	return INVALID_TOKEN;
 }
@@ -1205,10 +1233,11 @@ json_object_lookup_primitive(
 	const char     * key)
 {
 	tid = json_object_lookup(ctx, tid, key, JSMN_PRIMITIVE);
-	if (INVALID_TOKEN  != tid)
+	if (INVALID_TOKEN  != tid) {
 		return ctx->buf + ctx->tok[tid].start;
-	else
+	} else {
 		return NULL;
+	}
 }
 /* ------------------------------------------------------------------ */
 /* look up a boolean value. This essentially returns a tribool:
@@ -1271,8 +1300,9 @@ json_object_lookup_int(
 	cp = json_object_lookup_primitive(ctx, tid, key);
 	if (NULL != cp) {
 		ret = strtojint(cp, &ep);
-		if (cp != ep && '\0' == *ep)
+		if (cp != ep && '\0' == *ep) {
 			return ret;
+		}
 	} else {
 		errno = EINVAL;
 	}
@@ -1293,8 +1323,9 @@ json_object_lookup_int_default(
 	cp = json_object_lookup_primitive(ctx, tid, key);
 	if (NULL != cp) {
 		ret = strtojint(cp, &ep);
-		if (cp != ep && '\0' == *ep)
+		if (cp != ep && '\0' == *ep) {
 			return ret;
+		}
 	}
 	return def;
 }
@@ -1315,8 +1346,9 @@ json_object_lookup_float_default(
 	cp = json_object_lookup_primitive(ctx, tid, key);
 	if (NULL != cp) {
 		ret = strtod(cp, &ep);
-		if (cp != ep && '\0' == *ep)
+		if (cp != ep && '\0' == *ep) {
 			return ret;
+		}
 	}
 	return def;
 }
@@ -1398,8 +1430,9 @@ process_watch(
 	UNUSED_ARG(rtime);
 
 	path = json_object_lookup_string(jctx, 0, "device");
-	if (NULL == path || strcmp(path, up->device))
+	if (NULL == path || strcmp(path, up->device)) {
 		return;
+	}
 
 	if (json_object_lookup_bool(jctx, 0, "enable") > 0 &&
 	    json_object_lookup_bool(jctx, 0, "json"  ) > 0  )
@@ -1442,7 +1475,8 @@ process_version(
 	if (0 == errno) {
 		if ( ! up->fl_vers)
 			msyslog(LOG_INFO,
-				"REFCLOCK: %s: GPSD revision=%s release=%s protocol=%u.%u",
+				"REFCLOCK: %s: GPSD revision=%s release=%s "
+				"protocol=%u.%u",
 				up->logname, revision, release,
 				pvhi, pvlo);
 		up->proto_version = PROTO_VERSION(pvhi, pvlo);
@@ -1467,7 +1501,7 @@ process_version(
 	/* request watch for our GPS device if not yet watched.
 	 *
 	 * The version string is also sent as a life signal, if we have
-	 * seen useable data. So if we're already watching the device,
+	 * seen usable data. So if we're already watching the device,
 	 * skip the request.
 	 *
 	 * Reuse the input buffer, which is no longer needed in the
@@ -1491,8 +1525,9 @@ process_version(
 		 * resulting data timeout will take care of the
 		 * connection!
 		 */
-		msyslog(LOG_ERR, "REFCLOCK: %s: failed to write watch request (%m)",
-			up->logname);
+		msyslog(LOG_ERR,
+                        "REFCLOCK: %s: failed to write watch request (%s)",
+			up->logname, strerror(errno));
 	}
 }
 
@@ -1636,7 +1671,7 @@ process_pps(
 	 * primary channel. Sanity checks are done in evaluation step.
 	 */
 	up->pps_stamp = up->pps_recvt;
-	up->pps_stamp += 0x80000000u;
+	up->pps_stamp += 0x80000000U;
 	setlfpfrac(up->pps_stamp, 0);
 
 	if (NULL != up->pps_peer)
@@ -1733,18 +1768,19 @@ gpsd_parse(
 		return;
 	}
 
-	if      (!strcmp("TPV", clsid))
+	if      (!strcmp("TPV", clsid)) {
 		process_tpv(peer, &up->json_parse, rtime);
-	else if (!strcmp("PPS", clsid))
+	} else if (!strcmp("PPS", clsid)) {
 		process_pps(peer, &up->json_parse, rtime);
-	else if (!strcmp("TOFF", clsid))
+	} else if (!strcmp("TOFF", clsid)) {
 		process_toff(peer, &up->json_parse, rtime);
-	else if (!strcmp("VERSION", clsid))
+	} else if (!strcmp("VERSION", clsid)) {
 		process_version(peer, &up->json_parse, rtime);
-	else if (!strcmp("WATCH", clsid))
+	} else if (!strcmp("WATCH", clsid)) {
 		process_watch(peer, &up->json_parse, rtime);
-	else
+	} else {
 		return; /* nothing we know about... */
+	}
 	++up->tc_recv;
 
 	/* if possible, feed the PPS side channel */
@@ -1825,8 +1861,9 @@ gpsd_init_socket(
 	int          ov;
 
 	/* draw next address to try */
-	if (NULL == up->addr)
+	if (NULL == up->addr) {
 		up->addr = s_gpsd_addr;
+	}
 	ai = up->addr;
 	up->addr = ai->ai_next;
 
@@ -1836,8 +1873,8 @@ gpsd_init_socket(
 	if (-1 == up->fdt) {
 		if (syslogok(pp, up))
 			msyslog(LOG_ERR,
-				"REFCLOCK: %s: cannot create GPSD socket: %m",
-				up->logname);
+				"REFCLOCK: %s: cannot create GPSD socket: %s",
+				up->logname, strerror(errno));
 		goto no_socket;
 	}
 
@@ -1849,8 +1886,9 @@ gpsd_init_socket(
 	if (-1 == rc) {
 		if (syslogok(pp, up))
 			msyslog(LOG_ERR,
-				"REFCLOCK: %s: cannot set GPSD socket to non-blocking: %m",
-				up->logname);
+				"REFCLOCK: %s: cannot set GPSD socket "
+                                "to non-blocking: %s",
+				up->logname, strerror(errno));
 		goto no_socket;
 	}
 	/* Disable nagling. The way both GPSD and NTPD handle the
@@ -1865,8 +1903,8 @@ gpsd_init_socket(
 	if (-1 == rc) {
 		if (syslogok(pp, up))
 			msyslog(LOG_INFO,
-				"REFCLOCK: %s: cannot disable TCP nagle: %m",
-				up->logname);
+				"REFCLOCK: %s: cannot disable TCP nagle: %s",
+				up->logname, strerror(errno));
 	}
 
 	/* Start a non-blocking connect. There might be a synchronous
@@ -1882,8 +1920,8 @@ gpsd_init_socket(
 
 		if (syslogok(pp, up))
 			msyslog(LOG_ERR,
-				"REFCLOCK: %s: cannot connect GPSD socket: %m",
-				up->logname);
+				"REFCLOCK: %s: cannot connect GPSD socket: %s",
+				up->logname, strerror(errno));
 		goto no_socket;
 	}
 
@@ -1900,7 +1938,8 @@ gpsd_init_socket(
 	if (0 == io_addclock(&pp->io)) {
 		if (syslogok(pp, up))
 			msyslog(LOG_ERR,
-				"REFCLOCK: %s: failed to register with I/O engine",
+				"REFCLOCK: %s: failed to register "
+                                "with I/O engine",
 				up->logname);
 		goto no_socket;
 	}
@@ -1910,8 +1949,9 @@ gpsd_init_socket(
   no_socket:
 	if (-1 != pp->io.fd)
 		close(pp->io.fd);
-	if (-1 != up->fdt)
+	if (-1 != up->fdt) {
 		close(up->fdt);
+	}
 	pp->io.fd    = -1;
 	up->fdt      = -1;
 	up->tickover = up->tickpres;
@@ -1945,8 +1985,9 @@ gpsd_test_socket(
 		FD_ZERO(&wset);
 		FD_SET(up->fdt, &wset);
 		rc = pselect(up->fdt+1, NULL, &wset, NULL, &tout, NULL);
-		if (0 == rc || !(FD_ISSET(up->fdt, &wset)))
+		if (0 == rc || !(FD_ISSET(up->fdt, &wset))) {
 			return;
+		}
 	}
 
 	/* next timeout is a full one... */
@@ -1982,8 +2023,8 @@ gpsd_test_socket(
 	if (0 == io_addclock(&pp->io)) {
 		if (syslogok(pp, up))
 			msyslog(LOG_ERR,
-				"REFCLOCK: %s: failed to register with I/O engine",
-				up->logname);
+			    "REFCLOCK: %s: failed to register with I/O engine",
+			    up->logname);
 		goto no_socket;
 	}
 	return;
@@ -2010,10 +2051,12 @@ static int16_t
 clamped_precision(
 	int rawprec)
 {
-	if (rawprec > 0)
+	if (rawprec > 0) {
 		rawprec = 0;
-	if (rawprec < -32)
+	}
+	if (rawprec < -32) {
 		rawprec = -32;
+	}
 	return (int16_t)rawprec;
 }
 
@@ -2068,14 +2111,12 @@ save_ltc(
 	clockprocT * const pp,
 	const char * const tc)
 {
-	size_t len;
 
-	len = (tc) ? strlen(tc) : 0;
-	if (len >= sizeof(pp->a_lastcode))
-		len = sizeof(pp->a_lastcode) - 1;
-	pp->lencode = (unsigned short)len;
-	memcpy(pp->a_lastcode, tc, len);
-	pp->a_lastcode[len] = '\0';
+        if (NULL == tc) {
+	    pp->a_lastcode[0] = '\0';
+        } else {
+	    strlcpy(pp->a_lastcode, tc, sizeof(pp->a_lastcode));
+	}
 }
 
 /* -------------------------------------------------------------------
@@ -2097,8 +2138,9 @@ myasprintf(
 		alen += alen;
 		free(*spp);
 		*spp = (char*)malloc(alen);
-		if (NULL == *spp)
+		if (NULL == *spp) {
 			return -1;
+		}
 
 		va_start(va, fmt);
 		plen = (size_t)vsnprintf(*spp, alen, fmt, va);
@@ -2110,19 +2152,10 @@ myasprintf(
 
 /* -------------------------------------------------------------------
  * dump a raw data buffer
+ *
+ * Maybe this could be used system wide?
  */
 
-static char *
-add_string(
-	char *dp,
-	char *ep,
-	const char *sp)
-{
-	while (dp != ep && *sp)
-		*dp++ = *sp++;
-	return dp;
-}
-
 static void
 log_data(
 	peerT      *peer,
@@ -2130,8 +2163,7 @@ log_data(
 	const char *buf ,
 	size_t      len )
 {
-	/* we're running single threaded with regards to the clocks. */
-	static char s_lbuf[2048];
+	char s_lbuf[MAX_PDU_LEN];
 
 	clockprocT * const pp = peer->procptr;
 	gpsd_unitT * const up = (gpsd_unitT *)pp->unitptr;
@@ -2140,22 +2172,25 @@ log_data(
 		const char *sptr = buf;
 		const char *stop = buf + len;
 		char       *dptr = s_lbuf;
-		char       *dtop = s_lbuf + sizeof(s_lbuf) - 1; /* for NUL */
+                /* leave room for hex (\\x23) + NUL */
+		char       *dtop = s_lbuf + sizeof(s_lbuf) - 10;
 
-		while (sptr != stop && dptr != dtop) {
+		while (sptr != stop && dptr < dtop) {
 			if (*sptr == '\\') {
-				dptr = add_string(dptr, dtop, "\\\\");
+                                /* replace with two \ */
+				*dptr++ = '\\';
+				*dptr++ = '\\';
 			} else if (isprint(*sptr)) {
 				*dptr++ = *sptr;
 			} else {
-				char fbuf[6];
-				snprintf(fbuf, sizeof(fbuf), "\\%03o", *(const uint8_t*)sptr);
-				dptr = add_string(dptr, dtop, fbuf);
+                               dptr += snprintf(dptr, dtop - dptr, "\\%#.2x",
+                                                *(const uint8_t*)sptr);
+
 			}
 			sptr++;
 		}
 		*dptr = '\0';
-		mprintf("%s[%s]: '%s'\n", up->logname, what, s_lbuf);
+		printf("%s[%s]: '%s'\n", up->logname, what, s_lbuf);
 	}
 }
 
